'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _arrayJoinConjunction = require('array-join-conjunction');

var _arrayJoinConjunction2 = _interopRequireDefault(_arrayJoinConjunction);

var _checkerFactory = require('checker-factory');

var _checkerFactory2 = _interopRequireDefault(_checkerFactory);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = {
  get any() {
    return (0, _checkerFactory2.default)(undefined, 'any');
  },
  get array() {
    return (0, _checkerFactory2.default)(function (prop, key) {
      if (!Array.isArray(prop)) {
        var actualType = typeof prop === 'undefined' ? 'undefined' : _typeof(prop);
        return new TypeError('Expected ' + key + ' to be an `Array`, but got `' + actualType + '`');
      }
    }, 'array');
  },
  get arrayOf() {
    return function (validator) {
      return (0, _checkerFactory2.default)(function (prop, key) {
        if (!Array.isArray(prop)) {
          var actualType = typeof prop === 'undefined' ? 'undefined' : _typeof(prop);
          return new TypeError('Expected ' + key + ' to be an `Array`, but got `' + actualType + '`');
        }
        var anyErrors = prop.some(function (p) {
          return validator.validate(p) instanceof Error;
        });
        if (anyErrors) {
          return new TypeError(key + ' does not consist of the correct type');
        }
      }, 'arrayOf');
    };
  },
  get bool() {
    return (0, _checkerFactory2.default)('boolean', 'bool');
  },
  get func() {
    return (0, _checkerFactory2.default)('function', 'function');
  },
  get instanceOf() {
    return function (constructor) {
      return (0, _checkerFactory2.default)(function (prop, key) {
        if (!(prop instanceof constructor)) {
          /* eslint-disable prefer-reflect */
          var actualConstructorName = prop === null ? prop : Object.getPrototypeOf(prop).constructor.name;
          var errorMsg = 'Expected ' + key + ' to be an instance of `' + constructor.name + '`, ' + ('but got `' + actualConstructorName + '`');
          return new TypeError(errorMsg);
        }
      }, 'instanceOf');
    };
  },
  get number() {
    return (0, _checkerFactory2.default)('number', 'number');
  },
  get object() {
    return (0, _checkerFactory2.default)('object', 'object');
  },
  get objectOf() {
    return function (validator) {
      return (0, _checkerFactory2.default)(function (prop, key) {
        var propKeys = Object.keys(prop);

        for (var i = 0; i < propKeys.length; i++) {
          var validatorResult = validator.validate(prop[propKeys[i]], key + '.' + propKeys[i]);

          if (validatorResult instanceof Error) {
            return validatorResult;
          }
        }
      }, 'objectOf');
    };
  },
  get oneOf() {
    return function (allowedValues) {
      return (0, _checkerFactory2.default)(function (prop, key) {
        var isAllowed = allowedValues.some(function (value) {
          return value === prop;
        });
        if (!isAllowed) {
          var valuesMsg = (0, _arrayJoinConjunction2.default)(allowedValues.map(function (v) {
            return '`' + v + '`';
          }), 'or');
          var errMsg = 'Expected ' + key + ' to be ' + valuesMsg + ', but got `' + prop + '`';
          return new TypeError(errMsg);
        }
      }, 'oneOf');
    };
  },
  get oneOfType() {
    return function (allowedTypes) {
      return (0, _checkerFactory2.default)(function (prop, key) {
        var isAllowed = !allowedTypes.every(function (type) {
          return type.validate(prop) instanceof Error;
        });
        if (!isAllowed) {
          var typesMsg = (0, _arrayJoinConjunction2.default)(allowedTypes.map(function (t) {
            return '`' + t.name + '`';
          }), 'or');
          var errMsg = 'Expected ' + key + ' to be ' + typesMsg + ', but got `' + (typeof prop === 'undefined' ? 'undefined' : _typeof(prop)) + '`';
          return new TypeError(errMsg);
        }
      }, 'oneOfType');
    };
  },
  get shape() {
    return function (propsObj) {
      return (0, _checkerFactory2.default)(function (prop, key) {
        var validators = Object.keys(propsObj);

        for (var i = 0; i < validators.length; i++) {
          var validatorResult = propsObj[validators[i]].validate(prop[validators[i]], key + '.' + validators[i]);

          if (validatorResult instanceof Error) {
            return validatorResult;
          }
        }
      }, 'shape');
    };
  },
  get string() {
    return (0, _checkerFactory2.default)('string', 'string');
  }
};